@@ -2,22 +2,23 @@ from binascii import unhexlify
 from time import time
 
 from django import forms
-from django.forms import ModelForm, Form
-from django.utils.translation import ugettext_lazy as _
-
+from django.forms import Form, ModelForm
+from django.utils.translation import gettext_lazy as _
 from django_otp.forms import OTPAuthenticationFormMixin
 from django_otp.oath import totp
 from django_otp.plugins.otp_totp.models import TOTPDevice
-from two_factor.utils import totp_digits
+
+from .models import (
+    PhoneDevice, get_available_methods, get_available_phone_methods,
+)
+from .utils import totp_digits
+from .validators import validate_international_phonenumber
 
 try:
     from otp_yubikey.models import RemoteYubikeyDevice, YubikeyDevice
 except ImportError:
     RemoteYubikeyDevice = YubikeyDevice = None
 
-from .models import (PhoneDevice, get_available_phone_methods,
-                     get_available_methods)
-
 
 class MethodForm(forms.Form):
     method = forms.ChoiceField(label=_("Method"),
@@ -25,11 +26,13 @@ class MethodForm(forms.Form):
                                widget=forms.RadioSelect)
 
     def __init__(self, **kwargs):
-        super(MethodForm, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.fields['method'].choices = get_available_methods()
 
 
 class PhoneNumberMethodForm(ModelForm):
+    number = forms.CharField(label=_("Phone Number"),
+                             validators=[validate_international_phonenumber])
     method = forms.ChoiceField(widget=forms.RadioSelect, label=_('Method'))
 
     class Meta:
@@ -37,11 +40,15 @@ class PhoneNumberMethodForm(ModelForm):
         fields = 'number', 'method',
 
     def __init__(self, **kwargs):
-        super(PhoneNumberMethodForm, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.fields['method'].choices = get_available_phone_methods()
 
 
 class PhoneNumberForm(ModelForm):
+    # Cannot use PhoneNumberField, as it produces a PhoneNumber object, which cannot be serialized.
+    number = forms.CharField(label=_("Phone Number"),
+                             validators=[validate_international_phonenumber])
+
     class Meta:
         model = PhoneDevice
         fields = 'number',
@@ -55,7 +62,7 @@ class DeviceValidationForm(forms.Form):
     }
 
     def __init__(self, device, **args):
-        super(DeviceValidationForm, self).__init__(**args)
+        super().__init__(**args)
         self.device = device
 
     def clean_token(self):
@@ -66,7 +73,7 @@ class DeviceValidationForm(forms.Form):
 
 
 class YubiKeyDeviceForm(DeviceValidationForm):
-    token = forms.CharField(label=_("YubiKey"))
+    token = forms.CharField(label=_("YubiKey"), widget=forms.PasswordInput())
 
     error_messages = {
         'invalid_token': _("The YubiKey could not be verified."),
@@ -74,7 +81,7 @@ class YubiKeyDeviceForm(DeviceValidationForm):
 
     def clean_token(self):
         self.device.public_id = self.cleaned_data['token'][:-32]
-        return super(YubiKeyDeviceForm, self).clean_token()
+        return super().clean_token()
 
 
 class TOTPDeviceForm(forms.Form):
@@ -85,7 +92,7 @@ class TOTPDeviceForm(forms.Form):
     }
 
     def __init__(self, key, user, metadata=None, **kwargs):
-        super(TOTPDeviceForm, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.key = key
         self.tolerance = 1
         self.t0 = 0
@@ -135,6 +142,16 @@ class AuthenticationTokenForm(OTPAuthenticationFormMixin, Form):
     otp_token = forms.IntegerField(label=_("Token"), min_value=1,
                                    max_value=int('9' * totp_digits()))
 
+    otp_token.widget.attrs.update({'autofocus': 'autofocus'})
+
+    # Our authentication form has an additional submit button to go to the
+    # backup token form. When the `required` attribute is set on an input
+    # field, that button cannot be used on browsers that implement html5
+    # validation. For now we'll use this workaround, but an even nicer
+    # solution would be to move the button outside the `<form>` and into
+    # its own `<form>`.
+    use_required_attribute = False
+
     def __init__(self, user, initial_device, **kwargs):
         """
         `initial_device` is either the user's default device, or the backup
@@ -142,7 +159,7 @@ class AuthenticationTokenForm(OTPAuthenticationFormMixin, Form):
         be verified against all devices, it is not limited to the given
         device.
         """
-        super(AuthenticationTokenForm, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.user = user
 
         # YubiKey generates a OTP of 44 characters (not digits). So if the
@@ -150,7 +167,7 @@ class AuthenticationTokenForm(OTPAuthenticationFormMixin, Form):
         # IntegerField with a CharField.
         if RemoteYubikeyDevice and YubikeyDevice and \
                 isinstance(initial_device, (RemoteYubikeyDevice, YubikeyDevice)):
-            self.fields['otp_token'] = forms.CharField(label=_('YubiKey'))
+            self.fields['otp_token'] = forms.CharField(label=_('YubiKey'), widget=forms.PasswordInput())
 
     def clean(self):
         self.clean_otp(self.user)
